@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from typing import Dict, List, Optional
 
 from chia.consensus.block_record import BlockRecord
@@ -17,22 +19,27 @@ class BlockchainInterface:
         pass
 
     def block_record(self, header_hash: bytes32) -> BlockRecord:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     def height_to_block_record(self, height: uint32) -> BlockRecord:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     def get_ses_heights(self) -> List[uint32]:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     def get_ses(self, height: uint32) -> SubEpochSummary:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     def height_to_hash(self, height: uint32) -> Optional[bytes32]:
         pass
 
     def contains_block(self, header_hash: bytes32) -> bool:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     def remove_block_record(self, header_hash: bytes32) -> None:
         pass
@@ -41,7 +48,8 @@ class BlockchainInterface:
         pass
 
     def contains_height(self, height: uint32) -> bool:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     async def warmup(self, fork_point: uint32) -> None:
         pass
@@ -50,12 +58,14 @@ class BlockchainInterface:
         pass
 
     async def get_block_records_in_range(self, start: int, stop: int) -> Dict[bytes32, BlockRecord]:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     async def get_header_blocks_in_range(
         self, start: int, stop: int, tx_filter: bool = True
     ) -> Dict[bytes32, HeaderBlock]:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     async def get_header_block_by_height(
         self, height: int, header_hash: bytes32, tx_filter: bool = True
@@ -63,7 +73,8 @@ class BlockchainInterface:
         pass
 
     async def get_block_records_at(self, heights: List[uint32]) -> List[BlockRecord]:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
 
     def try_block_record(self, header_hash: bytes32) -> Optional[BlockRecord]:
         if self.contains_block(header_hash):
@@ -82,4 +93,5 @@ class BlockchainInterface:
         pass
 
     def seen_compact_proofs(self, vdf_info: VDFInfo, height: uint32) -> bool:
-        pass
+        # ignoring hinting error until we handle our interfaces more formally
+        return  # type: ignore[return-value]
