@@ -1,4 +1,7 @@
+from __future__ import annotations
+
 import asyncio
+import functools
 import json
 import logging
 import os
@@ -12,10 +15,10 @@ import uuid
 from concurrent.futures import ThreadPoolExecutor
 from enum import Enum
 from pathlib import Path
-from typing import Any, Dict, List, Optional, TextIO, Tuple, cast
+from typing import Any, Dict, List, Optional, TextIO, Tuple
 
 from chia import __version__
-from chia.cmds.init_funcs import check_keys, chia_init, chia_full_version_str
+from chia.cmds.init_funcs import check_keys, chia_full_version_str, chia_init
 from chia.cmds.passphrase_funcs import default_passphrase, using_default_passphrase
 from chia.daemon.keychain_server import KeychainServer, keychain_commands
 from chia.daemon.windows_signal import kill
@@ -23,16 +26,14 @@ from chia.plotters.plotters import get_available_plotters
 from chia.plotting.util import add_plot_directory
 from chia.server.server import ssl_context_for_root, ssl_context_for_server
 from chia.ssl.create_ssl import get_mozilla_ca_crt
-from chia.util.chia_logging import initialize_logging
+from chia.util.beta_metrics import BetaMetricsLogger
+from chia.util.chia_logging import initialize_service_logging
 from chia.util.config import load_config
-from chia.util.errors import KeychainRequiresMigration, KeychainCurrentPassphraseIsInvalid
+from chia.util.errors import KeychainCurrentPassphraseIsInvalid
 from chia.util.json_util import dict_to_json_str
-from chia.util.keychain import (
-    Keychain,
-    passphrase_requirements,
-    supports_os_passphrase_storage,
-)
+from chia.util.keychain import Keychain, passphrase_requirements, supports_os_passphrase_storage
 from chia.util.lock import Lockfile, LockfileError
+from chia.util.network import WebServer
 from chia.util.service_groups import validate_service
 from chia.util.setproctitle import setproctitle
 from chia.util.ws_message import WsRpcMessage, create_payload, format_response
@@ -141,13 +142,13 @@ class WebSocketServer:
         self.self_hostname = self.net_config["self_hostname"]
         self.daemon_port = self.net_config["daemon_port"]
         self.daemon_max_message_size = self.net_config.get("daemon_max_message_size", 50 * 1000 * 1000)
-        self.websocket_runner: Optional[web.AppRunner] = None
+        self.webserver: Optional[WebServer] = None
         self.ssl_context = ssl_context_for_server(ca_crt_path, ca_key_path, crt_path, key_path, log=self.log)
         self.keychain_server = KeychainServer()
         self.run_check_keys_on_unlock = run_check_keys_on_unlock
         self.shutdown_event = shutdown_event
 
-    async def start(self):
+    async def start(self) -> None:
         self.log.info("Starting Daemon Server")
 
         # Note: the minimum_version has been already set to TLSv1_2
@@ -169,28 +170,31 @@ class WebSocketServer:
                 ssl.OPENSSL_VERSION,
             )
 
-        def master_close_cb():
-            asyncio.create_task(self.stop())
+        self.webserver = await WebServer.create(
+            hostname=self.self_hostname,
+            port=self.daemon_port,
+            keepalive_timeout=300,
+            shutdown_timeout=3,
+            routes=[web.get("/", self.incoming_connection)],
+            ssl_context=self.ssl_context,
+            logger=self.log,
+        )
 
+    async def setup_process_global_state(self) -> None:
         try:
-            asyncio.get_running_loop().add_signal_handler(signal.SIGINT, master_close_cb)
-            asyncio.get_running_loop().add_signal_handler(signal.SIGTERM, master_close_cb)
+            asyncio.get_running_loop().add_signal_handler(
+                signal.SIGINT,
+                functools.partial(self._accept_signal, signal_number=signal.SIGINT),
+            )
+            asyncio.get_running_loop().add_signal_handler(
+                signal.SIGTERM,
+                functools.partial(self._accept_signal, signal_number=signal.SIGTERM),
+            )
         except NotImplementedError:
             self.log.info("Not implemented")
 
-        app = web.Application(client_max_size=self.daemon_max_message_size)
-        app.add_routes([web.get("/", self.incoming_connection)])
-        self.websocket_runner = web.AppRunner(app, access_log=None, logger=self.log, keepalive_timeout=300)
-        await self.websocket_runner.setup()
-
-        site = web.TCPSite(
-            self.websocket_runner,
-            host=self.self_hostname,
-            port=self.daemon_port,
-            shutdown_timeout=3,
-            ssl_context=self.ssl_context,
-        )
-        await site.start()
+    def _accept_signal(self, signal_number: int, stack_frame=None):
+        asyncio.create_task(self.stop())
 
     def cancel_task_safe(self, task: Optional[asyncio.Task]):
         if task is not None:
@@ -202,7 +206,9 @@ class WebSocketServer:
     async def stop(self) -> Dict[str, Any]:
         self.cancel_task_safe(self.ping_job)
         service_names = list(self.services.keys())
-        stop_service_jobs = [kill_service(self.root_path, self.services, s_n) for s_n in service_names]
+        stop_service_jobs = [
+            asyncio.create_task(kill_service(self.root_path, self.services, s_n)) for s_n in service_names
+        ]
         if stop_service_jobs:
             await asyncio.wait(stop_service_jobs)
         self.services.clear()
@@ -320,35 +326,33 @@ class WebSocketServer:
         elif command == "ping":
             response = await ping()
         elif command == "start_service":
-            response = await self.start_service(cast(Dict[str, Any], data))
+            response = await self.start_service(data)
         elif command == "start_plotting":
-            response = await self.start_plotting(cast(Dict[str, Any], data))
+            response = await self.start_plotting(data)
         elif command == "stop_plotting":
-            response = await self.stop_plotting(cast(Dict[str, Any], data))
+            response = await self.stop_plotting(data)
         elif command == "stop_service":
-            response = await self.stop_service(cast(Dict[str, Any], data))
+            response = await self.stop_service(data)
+        elif command == "running_services":
+            response = await self.running_services(data)
         elif command == "is_running":
-            response = await self.is_running(cast(Dict[str, Any], data))
+            response = await self.is_running(data)
         elif command == "is_keyring_locked":
             response = await self.is_keyring_locked()
         elif command == "keyring_status":
             response = await self.keyring_status()
         elif command == "unlock_keyring":
-            response = await self.unlock_keyring(cast(Dict[str, Any], data))
+            response = await self.unlock_keyring(data)
         elif command == "validate_keyring_passphrase":
-            response = await self.validate_keyring_passphrase(cast(Dict[str, Any], data))
-        elif command == "migrate_keyring":
-            response = await self.migrate_keyring(cast(Dict[str, Any], data))
+            response = await self.validate_keyring_passphrase(data)
         elif command == "set_keyring_passphrase":
-            response = await self.set_keyring_passphrase(cast(Dict[str, Any], data))
+            response = await self.set_keyring_passphrase(data)
         elif command == "remove_keyring_passphrase":
-            response = await self.remove_keyring_passphrase(cast(Dict[str, Any], data))
-        elif command == "notify_keyring_migration_completed":
-            response = await self.notify_keyring_migration_completed(cast(Dict[str, Any], data))
+            response = await self.remove_keyring_passphrase(data)
         elif command == "exit":
             response = await self.stop()
         elif command == "register_service":
-            response = await self.register_service(websocket, cast(Dict[str, Any], data))
+            response = await self.register_service(websocket, data)
         elif command == "get_status":
             response = self.get_status()
         elif command == "get_version":
@@ -371,8 +375,6 @@ class WebSocketServer:
         can_save_passphrase: bool = supports_os_passphrase_storage()
         user_passphrase_is_set: bool = Keychain.has_master_passphrase() and not using_default_passphrase()
         locked: bool = Keychain.is_keyring_locked()
-        needs_migration: bool = Keychain.needs_migration()
-        can_remove_legacy_keys: bool = False  # Disabling GUI support for removing legacy keys post-migration
         can_set_passphrase_hint: bool = True
         passphrase_hint: str = Keychain.get_master_passphrase_hint() or ""
         requirements: Dict[str, Any] = passphrase_requirements()
@@ -381,8 +383,6 @@ class WebSocketServer:
             "is_keyring_locked": locked,
             "can_save_passphrase": can_save_passphrase,
             "user_passphrase_is_set": user_passphrase_is_set,
-            "needs_migration": needs_migration,
-            "can_remove_legacy_keys": can_remove_legacy_keys,
             "can_set_passphrase_hint": can_set_passphrase_hint,
             "passphrase_hint": passphrase_hint,
             "passphrase_requirements": requirements,
@@ -440,54 +440,6 @@ class WebSocketServer:
         response: Dict[str, Any] = {"success": success, "error": error}
         return response
 
-    async def migrate_keyring(self, request: Dict[str, Any]) -> Dict[str, Any]:
-        if Keychain.needs_migration() is False:
-            # If the keyring has already been migrated, we'll raise an error to the client.
-            # The reason for raising an error is because the migration request has side-
-            # effects beyond copying keys from the legacy keyring to the new keyring. The
-            # request may have set a passphrase and indicated that keys should be cleaned
-            # from the legacy keyring. If we were to return early and indicate success,
-            # the client and user's expectations may not match reality (were my keys
-            # deleted from the legacy keyring? was my passphrase set?).
-            return {"success": False, "error": "migration not needed"}
-
-        success: bool = False
-        error: Optional[str] = None
-        passphrase: Optional[str] = request.get("passphrase", None)
-        passphrase_hint: Optional[str] = request.get("passphrase_hint", None)
-        save_passphrase: bool = request.get("save_passphrase", False)
-        cleanup_legacy_keyring: bool = request.get("cleanup_legacy_keyring", False)
-
-        if passphrase is not None and type(passphrase) is not str:
-            return {"success": False, "error": 'expected string value for "passphrase"'}
-
-        if passphrase_hint is not None and type(passphrase_hint) is not str:
-            return {"success": False, "error": 'expected string value for "passphrase_hint"'}
-
-        if not Keychain.passphrase_meets_requirements(passphrase):
-            return {"success": False, "error": "passphrase doesn't satisfy requirements"}
-
-        if type(cleanup_legacy_keyring) is not bool:
-            return {"success": False, "error": 'expected bool value for "cleanup_legacy_keyring"'}
-
-        try:
-            Keychain.migrate_legacy_keyring(
-                passphrase=passphrase,
-                passphrase_hint=passphrase_hint,
-                save_passphrase=save_passphrase,
-                cleanup_legacy_keyring=cleanup_legacy_keyring,
-            )
-            success = True
-            # Inform the GUI of keyring status changes
-            self.keyring_status_changed(await self.keyring_status(), "wallet_ui")
-        except Exception as e:
-            tb = traceback.format_exc()
-            self.log.error(f"Legacy keyring migration failed: {e} {tb}")
-            error = f"keyring migration failed: {e}"
-
-        response: Dict[str, Any] = {"success": success, "error": error}
-        return response
-
     async def set_keyring_passphrase(self, request: Dict[str, Any]) -> Dict[str, Any]:
         success: bool = False
         error: Optional[str] = None
@@ -519,8 +471,6 @@ class WebSocketServer:
                 passphrase_hint=passphrase_hint,
                 save_passphrase=save_passphrase,
             )
-        except KeychainRequiresMigration:
-            error = "keyring requires migration"
         except KeychainCurrentPassphraseIsInvalid:
             error = "current passphrase is invalid"
         except Exception as e:
@@ -561,32 +511,6 @@ class WebSocketServer:
         response: Dict[str, Any] = {"success": success, "error": error}
         return response
 
-    async def notify_keyring_migration_completed(self, request: Dict[str, Any]) -> Dict[str, Any]:
-        success: bool = False
-        error: Optional[str] = None
-        key: Optional[str] = request.get("key", None)
-
-        if type(key) is not str:
-            return {"success": False, "error": "missing key"}
-
-        Keychain.handle_migration_completed()
-
-        try:
-            if Keychain.master_passphrase_is_valid(key, force_reload=True):
-                Keychain.set_cached_master_passphrase(key)
-                success = True
-                # Inform the GUI of keyring status changes
-                self.keyring_status_changed(await self.keyring_status(), "wallet_ui")
-            else:
-                error = "bad passphrase"
-        except Exception as e:
-            tb = traceback.format_exc()
-            self.log.error(f"Keyring passphrase validation failed: {e} {tb}")
-            error = "validation exception"
-
-        response: Dict[str, Any] = {"success": success, "error": error}
-        return response
-
     def get_status(self) -> Dict[str, Any]:
         response = {"success": True, "genesis_initialized": True}
         return response
@@ -603,7 +527,7 @@ class WebSocketServer:
     async def _keyring_status_changed(self, keyring_status: Dict[str, Any], destination: str):
         """
         Attempt to communicate with the GUI to inform it of any keyring status changes
-        (e.g. keyring becomes unlocked or migration completes)
+        (e.g. keyring becomes unlocked)
         """
         websockets = self.connections.get("wallet_ui", None)
 
@@ -736,19 +660,17 @@ class WebSocketServer:
         p = request.get("p")  # Pool pubkey
         c = request.get("c")  # Pool contract address
 
-        command_args: List[str] = []
-        command_args.append(f"-n{n}")
-        command_args.append(f"-d{d}")
-        command_args.append(f"-r{r}")
+        command_args: List[str] = ["-n", str(n), "-d", d, "-r", str(r)]
 
         if f is not None:
-            command_args.append(f"-f{f}")
-
+            command_args.append("-f")
+            command_args.append(str(f))
         if p is not None:
-            command_args.append(f"-p{p}")
-
+            command_args.append("-p")
+            command_args.append(str(p))
         if c is not None:
-            command_args.append(f"-c{c}")
+            command_args.append("-c")
+            command_args.append(str(c))
 
         return command_args
 
@@ -763,38 +685,97 @@ class WebSocketServer:
         x = request["x"]  # Exclude final directory
         override_k = request["overrideK"]  # Force plot sizes < k32
 
-        command_args: List[str] = []
-        command_args.append(f"-k{k}")
-        command_args.append(f"-t{t}")
-        command_args.append(f"-2{t2}")
-        command_args.append(f"-b{b}")
-        command_args.append(f"-u{u}")
+        command_args: List[str] = ["-k", str(k), "-t", t, "-2", t2, "-b", str(b), "-u", str(u)]
 
         if a is not None:
-            command_args.append(f"-a{a}")
-
+            command_args.append("-a")
+            command_args.append(str(a))
         if e is True:
             command_args.append("-e")
-
         if x is True:
             command_args.append("-x")
-
         if override_k is True:
             command_args.append("--override-k")
 
         return command_args
 
     def _bladebit_plotting_command_args(self, request: Any, ignoreCount: bool) -> List[str]:
-        w = request.get("w", False)  # Warm start
-        m = request.get("m", False)  # Disable NUMA
+        plot_type = request["plot_type"]
+        assert plot_type == "ramplot" or plot_type == "diskplot"
 
         command_args: List[str] = []
 
-        if w is True:
-            command_args.append("-w")
+        if plot_type == "ramplot":
+            w = request.get("w", False)  # Warm start
+            m = request.get("m", False)  # Disable NUMA
+            no_cpu_affinity = request.get("no_cpu_affinity", False)
 
+            if w is True:
+                command_args.append("--warmstart")
+            if m is True:
+                command_args.append("--nonuma")
+            if no_cpu_affinity is True:
+                command_args.append("--no-cpu-affinity")
+
+            return command_args
+
+        # if plot_type == "diskplot"
+        w = request.get("w", False)  # Warm start
+        m = request.get("m", False)  # Disable NUMA
+        no_cpu_affinity = request.get("no_cpu_affinity", False)
+        # memo = request["memo"]
+        t1 = request["t"]  # Temp directory
+        t2 = request.get("t2")  # Temp2 directory
+        u = request.get("u")  # Buckets
+        cache = request.get("cache")
+        f1_threads = request.get("f1_threads")
+        fp_threads = request.get("fp_threads")
+        c_threads = request.get("c_threads")
+        p2_threads = request.get("p2_threads")
+        p3_threads = request.get("p3_threads")
+        alternate = request.get("alternate", False)
+        no_t1_direct = request.get("no_t1_direct", False)
+        no_t2_direct = request.get("no_t2_direct", False)
+
+        if w is True:
+            command_args.append("--warmstart")
         if m is True:
-            command_args.append("-m")
+            command_args.append("--nonuma")
+        if no_cpu_affinity is True:
+            command_args.append("--no-cpu-affinity")
+
+        command_args.append("-t")
+        command_args.append(t1)
+        if t2:
+            command_args.append("-2")
+            command_args.append(t2)
+        if u:
+            command_args.append("-u")
+            command_args.append(str(u))
+        if cache:
+            command_args.append("--cache")
+            command_args.append(str(cache))
+        if f1_threads:
+            command_args.append("--f1-threads")
+            command_args.append(str(f1_threads))
+        if fp_threads:
+            command_args.append("--fp-threads")
+            command_args.append(str(fp_threads))
+        if c_threads:
+            command_args.append("--c-threads")
+            command_args.append(str(c_threads))
+        if p2_threads:
+            command_args.append("--p2-threads")
+            command_args.append(str(p2_threads))
+        if p3_threads:
+            command_args.append("--p3-threads")
+            command_args.append(str(p3_threads))
+        if alternate:
+            command_args.append("--alternate")
+        if no_t1_direct:
+            command_args.append("--no-t1-direct")
+        if no_t2_direct:
+            command_args.append("--no-t2-direct")
 
         return command_args
 
@@ -828,6 +809,13 @@ class WebSocketServer:
         plotter: str = request.get("plotter", "chiapos")
         command_args: List[str] = ["chia", "plotters", plotter]
 
+        if plotter == "bladebit":
+            # plotter command must be either
+            # 'chia plotters bladebit ramplot' or 'chia plotters bladebit diskplot'
+            plot_type = request["plot_type"]
+            assert plot_type == "diskplot" or plot_type == "ramplot"
+            command_args.append(plot_type)
+
         command_args.extend(self._common_plotting_command_args(request, ignoreCount))
 
         if plotter == "chiapos":
@@ -888,7 +876,7 @@ class WebSocketServer:
             if state is not PlotState.SUBMITTED:
                 raise Exception(f"Plot with ID {id} has no state submitted")
 
-            id = config["id"]
+            assert id == config["id"]
             delay = config["delay"]
             await asyncio.sleep(delay)
 
@@ -1102,17 +1090,10 @@ class WebSocketServer:
         response = {"success": result, "service_name": service_name}
         return response
 
-    async def is_running(self, request: Dict[str, Any]) -> Dict[str, Any]:
-        service_name = request["service"]
-
+    def is_service_running(self, service_name: str) -> bool:
         if service_name == service_plotter:
             processes = self.services.get(service_name)
             is_running = processes is not None and len(processes) > 0
-            response = {
-                "success": True,
-                "service_name": service_name,
-                "is_running": is_running,
-            }
         else:
             process = self.services.get(service_name)
             is_running = process is not None and process.poll() is None
@@ -1122,23 +1103,32 @@ class WebSocketServer:
                 service_connections = self.connections.get(service_name)
                 if service_connections is not None:
                     is_running = len(service_connections) > 0
-            response = {
-                "success": True,
-                "service_name": service_name,
-                "is_running": is_running,
-            }
+        return is_running
 
-        return response
+    async def running_services(self, request: Dict[str, Any]) -> Dict[str, Any]:
+        services = list({*self.services.keys(), *self.connections.keys()})
+        running_services = [service_name for service_name in services if self.is_service_running(service_name)]
+
+        return {"success": True, "running_services": running_services}
+
+    async def is_running(self, request: Dict[str, Any]) -> Dict[str, Any]:
+        service_name = request["service"]
+        is_running = self.is_service_running(service_name)
+        return {"success": True, "service_name": service_name, "is_running": is_running}
 
     async def exit(self) -> None:
-        if self.websocket_runner is not None:
-            await self.websocket_runner.cleanup()
+        if self.webserver is not None:
+            self.webserver.close()
+            await self.webserver.await_closed()
         self.shutdown_event.set()
         log.info("chia daemon exiting")
 
     async def register_service(self, websocket: WebSocketResponse, request: Dict[str, Any]) -> Dict[str, Any]:
         self.log.info(f"Register service {request}")
-        service = request["service"]
+        service = request.get("service")
+        if service is None:
+            self.log.error("Service Name missing from request to 'register_service'")
+            return {"success": False}
         if service not in self.connections:
             self.connections[service] = []
         self.connections[service].append(websocket)
@@ -1331,7 +1321,7 @@ async def async_run_daemon(root_path: Path, wait_for_unlock: bool = False) -> in
     chia_init(root_path, should_check_keys=(not wait_for_unlock))
     config = load_config(root_path, "config.yaml")
     setproctitle("chia_daemon")
-    initialize_logging("daemon", config["logging"], root_path)
+    initialize_service_logging("daemon", config)
     crt_path = root_path / config["daemon_ssl"]["private_crt"]
     key_path = root_path / config["daemon_ssl"]["private_key"]
     ca_crt_path = root_path / config["private_ssl_ca"]["crt"]
@@ -1352,6 +1342,11 @@ async def async_run_daemon(root_path: Path, wait_for_unlock: bool = False) -> in
         with Lockfile.create(daemon_launch_lock_path(root_path), timeout=1):
             log.info(f"chia-blockchain version: {chia_full_version_str()}")
 
+            beta_metrics: Optional[BetaMetricsLogger] = None
+            if config.get("beta", {}).get("enabled", False):
+                beta_metrics = BetaMetricsLogger(root_path)
+                beta_metrics.start_logging()
+
             shutdown_event = asyncio.Event()
 
             ws_server = WebSocketServer(
@@ -1363,10 +1358,15 @@ async def async_run_daemon(root_path: Path, wait_for_unlock: bool = False) -> in
                 shutdown_event,
                 run_check_keys_on_unlock=wait_for_unlock,
             )
+            await ws_server.setup_process_global_state()
             await ws_server.start()
             await shutdown_event.wait()
+
+            if beta_metrics is not None:
+                await beta_metrics.stop_logging()
+
             log.info("Daemon WebSocketServer closed")
-            # sys.stdout.close()
+            sys.stdout.close()
             return 0
     except LockfileError:
         print("daemon: already launching")
