@@ -1,21 +1,23 @@
 # flake8: noqa: F811, F401
+from __future__ import annotations
+
 import asyncio
 import logging
 import time
 from typing import List
+from unittest.mock import MagicMock
 
 import pytest
 
-from chia.full_node.weight_proof import _validate_sub_epoch_summaries
 from chia.protocols import full_node_protocol
+from chia.simulator.block_tools import test_constants
+from chia.simulator.time_out_assert import time_out_assert
 from chia.types.blockchain_format.sub_epoch_summary import SubEpochSummary
 from chia.types.full_block import FullBlock
 from chia.types.peer_info import PeerInfo
 from chia.util.hash import std_hash
 from chia.util.ints import uint16
-from tests.core.node_height import node_height_exactly, node_height_between
-from tests.setup_nodes import test_constants
-from chia.simulator.time_out_assert import time_out_assert
+from tests.core.node_height import node_height_between, node_height_exactly
 
 log = logging.getLogger(__name__)
 
@@ -327,24 +329,43 @@ class TestFullSync:
         full_node_1, full_node_2, server_1, server_2, _ = two_nodes
         blocks = default_1000_blocks
 
+        # mock for full node _sync
+        async def async_mock():
+            log.info("do nothing")
+
+        full_node_2.full_node._sync = MagicMock(return_value=async_mock())
+
+        # load blocks into node 1
         for block in blocks[:501]:
             await full_node_1.full_node.respond_block(full_node_protocol.RespondBlock(block))
 
         peak1 = full_node_1.full_node.blockchain.get_peak()
-        full_node_2.full_node.sync_store.set_long_sync(True)
-        await server_2.start_client(PeerInfo(self_hostname, uint16(server_1._port)), full_node_2.full_node.on_connect)
-        wp = await full_node_1.full_node.weight_proof_handler.get_proof_of_weight(peak1.header_hash)
-        summaries1, _ = _validate_sub_epoch_summaries(full_node_1.full_node.weight_proof_handler.constants, wp)
-        summaries2 = summaries1
-        s = summaries1[1]
-        # change summary so check would fail on 2 sub epoch
-        summaries2[1] = SubEpochSummary(
+        assert peak1 is not None
+
+        summary_heights = full_node_1.full_node.blockchain.get_ses_heights()
+        summaries: List[SubEpochSummary] = []
+
+        # get ses list
+        for sub_epoch_n, ses_height in enumerate(summary_heights):
+            summaries.append(full_node_1.full_node.blockchain.get_ses(ses_height))
+
+        # change summary so check would fail on sub epoch 1
+        s = summaries[1]
+        summaries[1] = SubEpochSummary(
             s.prev_subepoch_summary_hash,
             s.reward_chain_hash,
             s.num_blocks_overflow,
             s.new_difficulty * 2,
             s.new_sub_slot_iters * 2,
         )
-        await full_node_2.full_node.sync_from_fork_point(0, 500, peak1.header_hash, summaries2)
-        log.info(f"full node height {full_node_2.full_node.blockchain.get_peak().height}")
-        assert node_height_between(full_node_2, 320, 400)
+        # manually try sync with wrong sub epoch summary list
+        await server_2.start_client(PeerInfo(self_hostname, uint16(server_1._port)), None)
+
+        # call peer has block to populate peer_to_peak
+        full_node_2.full_node.sync_store.peer_has_block(
+            peak1.header_hash, full_node_1.full_node.server.node_id, peak1.weight, peak1.height, True
+        )
+        # sync using bad ses list
+        await full_node_2.full_node.sync_from_fork_point(0, 500, peak1.header_hash, summaries)
+        # assert we failed somewhere between sub epoch 0 to sub epoch 1
+        assert node_height_between(full_node_2, summary_heights[0], summary_heights[1])
