@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import time
 from pathlib import Path
@@ -5,12 +7,15 @@ from typing import Any, Dict, List, Tuple
 
 import pytest
 from blspy import AugSchemeMPL, G1Element, G2Element
+
 from chia.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
 from chia.protocols.full_node_protocol import RespondBlock
 from chia.rpc.wallet_rpc_api import WalletRpcApi
 from chia.server.server import ChiaServer
+from chia.simulator.block_tools import BlockTools
 from chia.simulator.full_node_simulator import FullNodeSimulator
 from chia.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
+from chia.simulator.time_out_assert import time_out_assert, time_out_assert_not_none
 from chia.types.blockchain_format.program import Program
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.peer_info import PeerInfo
@@ -23,10 +28,7 @@ from chia.wallet.util.transaction_type import TransactionType
 from chia.wallet.util.wallet_types import AmountWithPuzzlehash
 from chia.wallet.wallet_node import WalletNode, get_wallet_db_path
 from chia.wallet.wallet_state_manager import WalletStateManager
-from chia.simulator.block_tools import BlockTools
-from chia.simulator.time_out_assert import time_out_assert, time_out_assert_not_none
 from tests.util.wallet_is_synced import wallet_is_synced
-from tests.wallet.cat_wallet.test_cat_wallet import tx_in_pool
 
 
 class TestWalletSimulator:
@@ -525,11 +527,7 @@ class TestWalletSimulator:
         assert tx_split_coins.spend_bundle is not None
 
         await wallet.push_transaction(tx_split_coins)
-        await time_out_assert(
-            15, tx_in_pool, True, full_node_1.full_node.mempool_manager, tx_split_coins.spend_bundle.name()
-        )
-        for i in range(0, num_blocks):
-            await full_node_1.farm_new_transaction_block(FarmNewBlockProtocol(bytes32(32 * b"0")))
+        await full_node_1.process_transaction_records(records=[tx_split_coins])
 
         funds = sum(
             [
@@ -542,43 +540,30 @@ class TestWalletSimulator:
         max_sent_amount = await wallet.get_max_send_amount()
 
         # 1) Generate transaction that is under the limit
-        under_limit_tx = None
-        try:
-            under_limit_tx = await wallet.generate_signed_transaction(
-                uint64(max_sent_amount - 1),
-                ph,
-                uint64(0),
-            )
-        except ValueError:
-            assert ValueError
+        transaction_record = await wallet.generate_signed_transaction(
+            uint64(max_sent_amount - 1),
+            ph,
+            uint64(0),
+        )
 
-        assert under_limit_tx is not None
+        assert transaction_record.amount == uint64(max_sent_amount - 1)
 
         # 2) Generate transaction that is equal to limit
-        at_limit_tx = None
-        try:
-            at_limit_tx = await wallet.generate_signed_transaction(
-                uint64(max_sent_amount),
-                ph,
-                uint64(0),
-            )
-        except ValueError:
-            assert ValueError
+        transaction_record = await wallet.generate_signed_transaction(
+            uint64(max_sent_amount),
+            ph,
+            uint64(0),
+        )
 
-        assert at_limit_tx is not None
+        assert transaction_record.amount == uint64(max_sent_amount)
 
         # 3) Generate transaction that is greater than limit
-        above_limit_tx = None
-        try:
-            above_limit_tx = await wallet.generate_signed_transaction(
+        with pytest.raises(ValueError):
+            await wallet.generate_signed_transaction(
                 uint64(max_sent_amount + 1),
                 ph,
                 uint64(0),
             )
-        except ValueError:
-            pass
-
-        assert above_limit_tx is None
 
     @pytest.mark.parametrize(
         "trusted",
@@ -782,7 +767,7 @@ class TestWalletSimulator:
 
     @pytest.mark.parametrize(
         "trusted",
-        [False],
+        [True, False],
     )
     @pytest.mark.asyncio
     async def test_address_sliding_window(
